<?php

/*
 * @file
 * These are the base class definitions define protocols for all ShowMe derived classes.
 *
 * This module defines four major types of objects:
 *
 *  ShowMeObject - Base class which supports common functionality
 *  ShowMePresentation - A presentation class which takes care of display when a link is clicked.
 *  ShowMeDataProvider - Provides both link and presentation raw data. 
 *  ShowMeMetadata - Defines metadata for communication between a data provider and a presentation.
 *
 * Naming conventions used in these classes are based on interpreting Drupal naming conventions as
 * well as some oop common sense.
 *
 * class names:
 *   FooBarBaz
 * public properties and methods:
 *   fooBarBaz()
 * protected properties and methods (accessible only to subclasses):
 *   foo_bar_baz()
 * internal inaccessible private properties and methods (part of the internal implementation):
 *   _foo_bar_baz()
 */

/*
 * A ShowMeObject is the base object which supports shared functionality, such as object keys.
 * It also handles mime-type support for subobjects.
 */

abstract class ShowMeObject {
  private $objkey;

  public function __construct($objkey_arg) {
    $this->objkey = $objkey_arg;

    // Assure that an objkey will be suitable for use in URLs.
    if (!preg_match('/^[-_a-zA-Z0-9]+(,[-_a-zA-Z0-9]+)+$/', $objkey_arg)) {
      trigger_error("showme_api ShowMeObject: object key contains invalid characters: ".$objkey_arg);
    }
  }

  /*
   * Return the variant associated with this object (the second part of the objkey)
   */

  public function getVariant() {
    $parts = explode(',', $this->objkey, 2);
    return $parts[1];
  }

  /*
   * Return the object key
   */

  public function getObjkey() {
    return $this->objkey;
  }

  /*
   * Fetch the title of this object.   By default, returns the object key. Of course this is a 
   * terrible title to put in front of the user, but at least it reveals that you've forgotton to 
   * implement a good title.  This could have been abstract, but there may be cases where titles 
   * never appear in the future.
   */

  public function getTitle() {
    return '('.$this->objkey.')';
  }

  /**
   * Returns the list of supported mime types or mime type templates (text/*)
   */

  public function supportedTypeList() {
    return array('text/html');
  }

  /**
   * Returns TRUE if any of the given mime types are supported by this presentation.
   */

  public function supportsMimeType($checktype) {

    if (is_array($checktype)) {
      foreach ($checktype as $onetype) {
        if ($this->supportsMimeType($onetype))
          return TRUE;
      }
      return FALSE;
    }

    $checkparts = explode('/', $checktype);

    if (count($checkparts) != 2)
      return FALSE;

    foreach ($this->supportedTypeList() as $type) {

      if ($type == $checktype)
        return TRUE;

      $parts = explode('/', $type);

      if (count($parts) == 2) {
        if ($parts[0] == $checkparts[0] && ($parts[1] == '*' || $checkparts[1] == '*'))
          return TRUE;
      }

    }

    return FALSE;
  }
}

/*
 * A ShowMePresentation is an object which is completely responsible for the display of a given
 * pop-up technology or media delivery when links to that media are clicked.
 * 
 * Examples of presentations:
 *      - Lightbox-style presentations for overlaid data
 *      - Simple targetted windows
 *      - Audio presentations which play a sound when a link is clicked
 */

abstract class ShowMePresentation extends ShowMeObject {

  /*
   * Returns the fully rendered link ready for page insertion
   */

  abstract public function renderSourceLinkFromElement($element, $dataobj);
}


/*
 * A ShowMeDataProvider is an object which renders data both for links to presentations and for
 * presentations itself.  It has all responsibility for the format of the data and only provides
 * metadata (such as preferred sizes, etc.) for use by presentations.
 *
 * Example of data provider:
 *      - A drupal URL on the current site which displays text/html information
 *      - A path to an image/jpg or image/* which is a directly accessible URL on this or another site
 */

abstract class ShowMeDataProvider extends ShowMeObject {

  /*
   * Return the supported field types.  Must be implemented.
   */

  abstract public function supportedFieldTypes();

  public function hasFieldSupport() {
    return !!$this->supportedFieldTypes();
  }

  /*
   * Returns Metadata given a node, fieldname and delta.  By default creates an element and then
   * retrieves the metadata using getMetadataFromElement().   NOTE this function is 'final' so that
   * it is not overloaded.   getMetadataFromElement should be overloaded, as it populates some
   * standard metadata fields.
   */

  public final function getMetadata($node, $field_name, $delta) {
    if ( ($element = _get_field_element($node, $field_name, $delta)) ) {
      return getMetadataFromElement($element);
    }
  }

  /*
   * This returns metadata (title, suggested size, alt, etc.) for a given element.  
   * SUBCLASSES: call your parent to save time and energy in getting the basic metadata info.
   */

  public function getMetadataFromElement($element) {
    $node = $element['#node'];

    $metadata = new ShowMeMetadata();

    $metadata->node = $node;
    $metadata->title = $node->title;
    $metdata->alt = t('View: ' . $node->title);

    return $metadata;
  }

  // ========================================
  //  Source rendering methods
  // ========================================

  /*
   * Returns the source content in HTML format.  By default, calls renderTargetFromElement specifying the
   * text/html mime type.
   *
   * DEFAULT IMPLEMENTATION: Source content will be the same as the target content.
   */

  public function renderSourceHtmlFromElement($element) {
    return $this->renderTargetFromElement($element, 'text/html');
  }

  /*
   * Returns the list of provided field types.  Returning an empty list assures that this object
   * will not be used in contexts where field support is expected.
   */

  // ========================================
  //  Target rendering methods
  // ========================================

  /*
   * renderTarget() SHOULD be overridden by classes which have internal knowledge of their data types.   
   * This implementation is wasteful, in that it builds the entire node just in order to assure that an
   * $element exists in order to call a single theme function.
   */

  public function renderTarget($node, $field_name, $delta, $mimetype) {
    if ( ($element = $this->_get_field_element($node, $field_name, $delta)) ) {
      return $this->renderTargetFromElement($element, 'text/html');
    }
  }

  /*
   * Renders the data as appropriate for the target presentation.  Technically this is the ONLY
   * rendering method which *must* be implemented, the rest have defaults derived from this one.
   */

  abstract public function renderTargetFromElement($element, $mime_type);

  /**
   * Returns the URL which the presentation will use to display the data.  The mime type requested
   * must be one of the ones listed in the supportedTypeList and the field element must be one
   * of those in the supportedFieldTypes list.
   *
   * The default implementation creates a drupal URL for a page which will extract the field value
   * from the given node and reconstruct the data provider object,
   */

  public function getTargetURL($element, $mimetype) {
    $node = $element['#node'];
    $item = $element['#item'];

    return 'showme_api/target_node/' . $node->nid . '/' .
      $element['#field_name'] . ',' . $item['#delta'] . ',' . $this->getObjkey();
  }

  // ========================================
  //  Utilities
  // ========================================

  /*
   * Private function to extract an element from a field.   Needed only if a field needs to be derived
   * from a triggered URL which results in a page being displayed other than the one in which the original
   * link was embedded.
   */

  private function _get_field_element($node, $field_name, $delta) {

    $field = content_fields($field_name);

    if ($field) {

      // We build all node content, then fetch only one element.  It would be nice if we could use
      // content_view_field.  However, that would result in calling the very field formatter which
      // ultimately got us here!  Instead, we need to delegate special formatting for original links
      // and popups to our classes, which are essentially "formatters on top of formatters".

      if (!isset($node->build_mode))
        node_build_content($node);

      $element = $node->content[$field_name]['field']['items'][$delta];

      // The location of the $element is different if there are field groups.  There really should be
      // a cck api for this, but this is lifted from content.module to find the field element in the
      // case of field groups.  Scary to think of upgrading this to D7.

      if (!$element && module_exists('fieldgroup') 
          && ($group_name = fieldgroup_get_group($node->type, $field_name))) {
        $element = $node->content[$group_name]['group'][$field_name]['field']['items'][$delta];
      }

      return $element;
    }

  }

}

/*
 * Metadata is defined here with fixed meanings so that various add-ons can agree upon
 * exactly what the meaning of various features, flags, and attributes are.  I know this isn't
 * "The Drupal Way" and many people probably would want a key->value array.  So sue me.
 *
 * NOTE: that Metadata is OPTIONAL and may not be available for any or all of the items.
 *
 */

class ShowMeMetadata {

  // Reference information about the containing object

  public $node;                 // Node object

  // Labeling information

  public $title;                // Title of the presented object
  public $alt;                  // Alt information for links or images

  // EXACT height and width of presentation space needed, as an integer, in pixels.  Don't include padding,
  // that's up to the presentation to deal with.

  public $height;
  public $width;

  // I lied.  If there really is weird stuff to tag along, then put it in here, but be warned.

  public $misc = array();       // The catch-all bin.

};
